library miodi;

import 'dart:collection';

import 'package:miodi/core/component.dart';
import 'package:miodi/core/module.dart';
import 'package:miodi/core/providers.dart';

export 'package:miodi/core/component.dart';

T inject<T>([String componentNamespace]) => Miodi._provide(componentNamespace);

Future<T> injectAsync<T>([String componentNamespace]) =>
    Miodi._provideAsync(componentNamespace);

Provider<T> singleton<T>(Type T, Creator creator) =>
    SingletonProvider(T, creator);

Provider<T> factory<T>(Type T, Creator creator) => FactoryProvider(T, creator);

MiodiModule module(List<Provider> providers) => MiodiModule(providers);

class Miodi {
  static T _provide<T>([String componentNamespace]) {
    T obj;
    if (componentNamespace == null) {
      for (String namespace in _namespaces) {
        if (_configuration[namespace].containsKey(T)) {
          obj = _configuration[namespace][T].provide() as T;
          break;
        }
      }
    } else {
      obj = _configuration[componentNamespace][T].provide() as T;
    }
    return obj;
  }

  Miodi._();

  static Miodi _instance;
  static final HashMap<String, HashMap<Type, Provider>> _configuration =
      HashMap();
  static final HashSet<String> _namespaces = HashSet();

  static void init(List<MiodiComponent> components) {
    if (_instance != null) return;
    components.forEach((component) {
      var componentNamespace = component.componentNamespace;
      _namespaces.add(componentNamespace);
      if (_configuration.containsKey(componentNamespace))
        _configuration[componentNamespace]
            .addAll(component.componentConfiguration);
      else _configuration[componentNamespace] = component.componentConfiguration;
    });
    _instance = Miodi._();
  }

  static Future<T> _provideAsync<T>([String componentNamespace]) async {
    return _provide<T>(componentNamespace);
  }
}
